Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' ColorCalibrator: this class handles calibration of a color set
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (02/10/05)    Initial creation of the class.
'
'#############################################################################################
Public Class ColorCalibrator
    Public X1 As Integer
    Public Y1 As Integer
    Public X2 As Integer
    Public Y2 As Integer

    Private bmImage As Bitmap


    '#########################################################################################
    'create a new reference; always point to correct image
    '#########################################################################################
    Sub New(ByRef bImage As Bitmap)
        bmImage = bImage
    End Sub


    '#########################################################################################
    'generate statistics about the color set captured here.
    '#########################################################################################
    Public Function ProcessSet() As String
        Dim iX1 As Integer
        Dim iY1 As Integer
        Dim iX2 As Integer
        Dim iY2 As Integer

        Dim I As Integer
        Dim J As Integer
        Dim iLen As Integer
        Dim iCount As Integer = 0

        Dim cColor As Color
        Dim mImage As RGBImage
        Dim mCalc As YUVImage

        Dim iY() As Integer
        Dim iU() As Integer
        Dim iV() As Integer

        Dim sOut As String

        'switch the start and end values, depending on selections.
        If X1 <= X2 Then
            iX1 = X1
            iX2 = X2
        Else
            iX1 = X2
            iX2 = X1
        End If

        'switch the start and end values, depending on selections.
        If Y1 <= Y2 Then
            iY1 = Y1
            iY2 = Y2
        Else
            iY1 = Y2
            iY2 = Y1
        End If

        'fill an RGBImage object with the subset of pixels selected from the image
        mImage = New RGBImage(1, 1)

        'create lists which can be analyzed
        iLen = (iX2 - iX1 + 1) * (iY2 - iY1 + 1)
        ReDim iY(iLen - 1)
        ReDim iU(iLen - 1)
        ReDim iV(iLen - 1)

        'loop through all pixels in the subset
        For I = iX1 To iX2
            For J = iY1 To iY2

                'get the color signal for current pixel
                cColor = bmImage.GetPixel(I, J)
                mImage.R(0, 0) = cColor.R
                mImage.G(0, 0) = cColor.G
                mImage.B(0, 0) = cColor.B

                'next, convert the image subset into YUV format for processing
                mCalc = mImage.ToYUV

                'now, add each pixel to each array
                iY(iCount) = mCalc.Y(0, 0)
                iU(iCount) = mCalc.U(0, 0)
                iV(iCount) = mCalc.V(0, 0)
                iCount += 1
            Next
        Next

        'three arrays containing information about this data are now ready.
        'analyze them.
        sOut = "Coordinates:   (" & iX1 & ", " & iY1 & ") to (" & iX2 & ", " & iY2 & ")" & vbCrLf & vbCrLf
        sOut &= "Stat:   (Y, U, V)" & vbCrLf

        'get the mean of each array and return it
        sOut &= "Mean:   ("
        sOut &= Format(nfMath.Basic.Average(iY), "0.00") & ", "
        sOut &= Format(nfMath.Basic.Average(iU), "0.00") & ", "
        sOut &= Format(nfMath.Basic.Average(iV), "0.00") & ")" & vbCrLf

        'get the deviation of each array and return it
        sOut &= "StDev:  ("
        sOut &= Format(nfMath.Basic.StandardDeviation(iY), "0.00") & ", "
        sOut &= Format(nfMath.Basic.StandardDeviation(iU), "0.00") & ", "
        sOut &= Format(nfMath.Basic.StandardDeviation(iV), "0.00") & ")" & vbCrLf

        'get the median of each array and return it
        sOut &= "Median: ("
        sOut &= Format(iY(iLen >> 1), "0.00") & ", "
        sOut &= Format(iU(iLen >> 1), "0.00") & ", "
        sOut &= Format(iV(iLen >> 1), "0.00") & ")" & vbCrLf

        Return sOut

    End Function

End Class
