Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' ColorImage: a simple base class for color images in any color space.
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (02/08/05)    Initial creation of the class.
'
'#############################################################################################

Public MustInherit Class ColorImage
    Protected imWidth As Integer = 0
    Protected imHeight As Integer = 0


    '#########################################################################################
    'create a new image. all color arrays have the same size and depth
    '#########################################################################################
    Sub New(ByVal iWidth As Integer, ByVal iHeight As Integer)

        imWidth = iWidth
        imHeight = iHeight
        Me.Resize()

    End Sub


    '#########################################################################################
    'resize the image. data is never preserved during a resize. must be overridden
    '#########################################################################################
    Protected MustOverride Sub Resize()


    '#########################################################################################
    'adjust width. changing width destroys the current image.
    '#########################################################################################
    Public Property Width() As Integer

        Get
            Return imWidth
        End Get

        Set(ByVal Value As Integer)
            imWidth = Value
            Me.Resize()
        End Set

    End Property


    '#########################################################################################
    'adjust width. changing width destroys the current image.
    '#########################################################################################
    Public Property Height() As Integer

        Get
            Return imHeight
        End Get

        Set(ByVal Value As Integer)
            imHeight = Value
            Me.Resize()
        End Set

    End Property


    '#########################################################################################
    'constrain any value to an unsigned byte between 0 and 255.
    '#########################################################################################
    Public Shared Function Constrain(ByVal dVal As Double) As Byte
        If dVal > 255 Then Return 255
        If dVal < 0 Then Return 0
        Return CByte(dVal)
    End Function


    '#########################################################################################
    'translate the selected color by the value, constraining it in the process
    '#########################################################################################
    Public Shared Function Translate(ByVal bColor As Byte, ByVal dOffset As Double) As Byte
        Return CByte(Constrain(bColor + dOffset))
    End Function


    '#########################################################################################
    'translate the selected color by the value, constraining it in the process
    '#########################################################################################
    Public Shared Function Scale(ByVal bColor As Byte, ByVal dOffset As Double) As Byte
        Return CByte(Constrain(bColor * dOffset))
    End Function


    '#########################################################################################
    'convert the passed-in array into an intensity bitmap.
    '#########################################################################################
    Public Shared Function ToIntensityBitmap(ByVal yArray(,) As Integer) As Bitmap
        Dim I As Integer
        Dim J As Integer
        Dim bImg As New Bitmap(yArray.GetLength(0), yArray.GetLength(1))

        'loop through every pixel in this image
        For I = 0 To yArray.GetLength(0) - 1
            For J = 0 To yArray.GetLength(1) - 1

                'save each pixel in its component format
                bImg.SetPixel(I, J, Color.FromArgb(yArray(I, J), yArray(I, J), yArray(I, J)))
            Next J
        Next I

        'return the completed image.
        Return bImg

    End Function

End Class
