Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' ColorMap: a
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (01/27/05)    Began work on the class.
'
'#############################################################################################


Public Class ColorMap
    'this collection contains the entire set of segmentable colors. each point will be put through
    'the probability analysis in this collection.
    Public Colors As New Collection


    '#########################################################################################
    'given a collection of colors to isolate and the red-blue chrominance maps of an image, 
    'convert the chrominance arrays into a complement: first array will contain a map with 
    'each element corresponding to the strongest color, while the second array will contain 
    'intensity of that particular color at that pixel (where 1 is a perfect match, 0 is no 
    'match)
    '#########################################################################################
    Public Sub Map(ByVal fYUV As YUVImage)
        Dim I As Integer
        Dim J As Integer
        Dim fColor As FuzzyColor
        Dim dColor As Double
        Dim dIntensity As Double
        Dim dCur As Double

        'loop through every single pixel in the map
        For I = 0 To fYUV.Width - 1
            For J = 0 To fYUV.Height - 1

                'reset current color and intensity
                dColor = 0
                dIntensity = 0

                'check the current pixel against all color probabilities
                For Each fColor In Colors
                    'get current probability. if it's better than current, then it replaces
                    'the current value.
                    dCur = fColor.Probability(fYUV.Y(I, J), fYUV.U(I, J), fYUV.V(I, J))
                    If dCur > dIntensity Then
                        dColor = fColor.ID
                        dIntensity = dCur
                    End If
                Next

                'once all colors have been checked, write the correct values to the array.
                dU(I, J) = dColor
                dV(I, J) = dIntensity
            Next
        Next

        'the color map is now complete and ready for interpretation.

    End Sub









End Class
