Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' Macro: a single input event, recorded as an XML fragment. the event is defined in derived
' code. the only part of the event common across all items is the time (from when the macro
' was initiated) at which the event occurred.
'
' <macro>
'   <time>0.05</time>
' </macro>
'
' this class derives a simple root class into a unique class with its own text tags.
'
' Interfaces Required:  none
' Classes Required:     xmlRoot
' Modules Required:     none 
' Other Requirements:   none
'
' Certification:        NOT In Progress (06/xx/04)
'                       Module Active in 0 Projects
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (06/09/04)    Initial creation of the class as the first unit in a dynamically-writable,
'               macro-recording hierarchy.
'
'#############################################################################################
Namespace naXML

    Public Class Macro
        Inherits naXML.xmlRoot

        Private dmTime As Double = 0.0


        '#####################################################################################
        'set/get the time from the start of macro recording that this event happened
        '#####################################################################################
        Public Overridable Property Time() As Double

            Get
                Return dmTime
            End Get

            Set(ByVal Value As Double)
                dmTime = Value
            End Set

        End Property


        '#####################################################################################
        'taking as input a single settings node structure, add this information to this 
        'setting
        '#####################################################################################
        Public Overrides Sub xmlRead(ByRef oNode As Xml.XmlElement)
            Dim oSubNode As Xml.XmlElement

            'always call the base class first
            MyBase.xmlRead(oNode)

            'use this structure to add optional nodes for example, tag is an optional node,
            'and will only be added if it is not default
            For Each oSubNode In oNode.ChildNodes

                Select Case oSubNode.Name.ToLower

                    Case "time"
                        Time = CDbl(oSubNode.InnerText)

                End Select

            Next oSubNode

        End Sub


        '#####################################################################################
        'using this item's settings and a provided xml document, output an element
        '#####################################################################################
        Public Overrides Function xmlWrite(ByRef xOut As Xml.XmlDocument) As Xml.XmlElement
            Dim xBase As Xml.XmlElement

            'create the root element; no attributes at this point, then add the id as the 
            'root attribute
            xBase = MyBase.xmlWrite(xOut)

            'create the tag and property element; only add it if it is not default (empty)
            xBase.AppendChild(xOut.CreateElement("time"))
            xBase("time").InnerText = CStr(Me.Time).Format("0.000")

            Return xBase

        End Function

    End Class

End Namespace
