Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' MacroTower:
'
' Interfaces Required:  none
' Classes Required:     RankedArray
' Modules Required:     none 
' Other Requirements:   none
'
' Certification:        In Progress (02/18/04)
'                       Module Active in 2 Projects
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (05/12/04)    MILESTONE: class shipped with project ASC DrawMove (internal).
'
'#############################################################################################

Namespace naXML

    Public Class MacroBuilder
        Inherits naXML.RankedArray

        Private Const XML_ROOT_NODE As String = "macrobuilder"


        '#####################################################################################
        'add an existing object to the collection. when an item is added, it's automatically 
        'added to the back of the class (fitting with rankedarray policy). in addition, a
        'key is auto-generated.
        '#####################################################################################
        Public Overrides Sub Add(ByVal sKey As String, ByRef oNew As naXML.xmlKeyed)

            'implement the base class
            MyBase.Add(sKey, oNew)

        End Sub


        '#####################################################################################
        'delete an item from a collection
        '#####################################################################################
        Public Overrides Sub Remove(ByVal sKey As String)
            MyBase.Remove(sKey)
            omRank.Remove(sKey)
        End Sub


        '#####################################################################################
        'remove all items from the hash table
        '#####################################################################################
        Public Overrides Sub Clear()
            omItems.Clear()
            omRank.Clear()
        End Sub


        '#####################################################################################
        'enumerator declaration so For Each..Next will work. this class must use the special
        'RankedWeak enumerator to ensure that the array is enumerated in the correct order
        '#####################################################################################
        Public Overrides Function GetEnumerator() As IEnumerator
            Return CType(New naXML.RankedWeakEnumerator(omItems, omRank), IEnumerator)
        End Function


        '#####################################################################################
        'this function copies the object in the parameter list and functions like a copy 
        'constructor
        '#####################################################################################
        Public Overrides Function Clone() As Object
            Dim oCopy As New naXML.RankedArray
            Dim oEnum As naXML.xmlKeyed
            Dim oRank As DictionaryEntry

            'execute a memberwise clone without cloning the member hash table (which seems to 
            'clone a reference)
            oCopy.Root = MyBase.Root
            oCopy.ID = MyBase.ID
            oCopy.Enabled = MyBase.Enabled
            oCopy.Tag = MyBase.Tag
            oCopy.Text = MyBase.Text
            oCopy.Visible = MyBase.Visible

            'empty out the hash table
            oCopy.Clear()

            'now clone each member item. a nice feature of this routine is that because we're
            'iterating through the array in its ranked order, we don't need to rebuild the
            'rank table in order to correctly rank them in the cloned array!
            For Each oEnum In Me
                oCopy.Add(oEnum.ID, CType(oEnum.Clone(), naXML.xmlKeyed))
            Next

            Return oCopy

        End Function


        '#####################################################################################
        'reverse the order of two items, switching them in their places. this is the primary
        'function for altering the order of the items in this array.
        '#####################################################################################
        Public Sub Transpose(ByVal sSrc As String, ByVal sDst As String)
            Dim lTemp As Long

            lTemp = CLng(omRank(sSrc))
            omRank(sSrc) = omRank(sDst)
            omRank(sDst) = lTemp

        End Sub


        '#####################################################################################
        'class initialization
        '#####################################################################################
        Public Sub New()
            MyBase.New()
            Root = XML_ROOT_NODE
        End Sub


        Public Sub New(ByVal sID As String)
            MyBase.New()
            MyBase.ID = sID
            Root = XML_ROOT_NODE
        End Sub


        Public Sub New(ByRef oNode As Xml.XmlElement)
            MyBase.New()
            xmlRead(oNode)
            Root = XML_ROOT_NODE
        End Sub

    End Class

End Namespace
