Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' YUVImage: a container for an array of YUV colors.
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (02/08/05)    Initial creation of the class.
'
'#############################################################################################

Public Class RGBImage
    Inherits ColorImage

    Public R(,) As Integer
    Public G(,) As Integer
    Public B(,) As Integer


    '#########################################################################################
    'create a new image. all color arrays have the same size and depth
    '#########################################################################################
    Sub New(ByVal iWidth As Integer, ByVal iHeight As Integer)
        MyBase.New(iWidth, iHeight)
    End Sub


    '#########################################################################################
    'resize the image. data is never preserved during a resize.
    '#########################################################################################
    Protected Overrides Sub Resize()

        ReDim R(imWidth - 1, imHeight - 1)
        ReDim G(imWidth - 1, imHeight - 1)
        ReDim B(imWidth - 1, imHeight - 1)

    End Sub


    '#########################################################################################
    'return the intensity of the passed-in pixel. 1 is white; 0 is black.
    '#########################################################################################
    Public Function Intensity(ByVal I As Integer, ByVal J As Integer) As Double
        Return 1 - ((R(I, J) + G(I, J) + B(I, J)) / 765)
    End Function


    '#########################################################################################
    'convert the passed-in bitmap into this RGBImage.
    '#########################################################################################
    Public Sub FromBitmap(ByRef bImg As Bitmap)
        Dim I As Integer
        Dim J As Integer

        'resize this object to match the bitmap
        imWidth = bImg.Width
        imHeight = bImg.Height
        Me.Resize()

        'loop through every pixel in this image
        For I = 0 To Width - 1
            For J = 0 To Height - 1

                'save each pixel in its component format
                R(I, J) = bImg.GetPixel(I, J).R
                G(I, J) = bImg.GetPixel(I, J).G
                B(I, J) = bImg.GetPixel(I, J).B
            Next J
        Next I

    End Sub


    '#########################################################################################
    'loop through all members of this array and convert it to an RGB image.
    '#########################################################################################
    Public Function ToYUV() As YUVImage
        Dim mYUV As YUVImage
        Dim I As Integer
        Dim J As Integer

        'create the new RGB image to return
        mYUV = New YUVImage(Width, Height)

        'loop through every pixel in this image
        For I = 0 To Width - 1
            For J = 0 To Height - 1

                'convert the standard RGB data into Y Cr Cb data in separate arrays
                mYUV.Y(I, J) = Constrain(((66 * Me.R(I, J) + 129 * Me.G(I, J) + 25 * Me.B(I, J) + 128) >> 8) + 16)
                mYUV.U(I, J) = Constrain(((-38 * Me.R(I, J) - 74 * Me.G(I, J) + 112 * Me.B(I, J) + 128) >> 8) + 128)
                mYUV.V(I, J) = Constrain(((112 * Me.R(I, J) - 94 * Me.G(I, J) - 18 * Me.B(I, J) + 128) >> 8) + 128)
            Next J
        Next I

        'return the result
        Return mYUV

    End Function


    '#########################################################################################
    'convert the current RGBImage into a bitmap.
    '#########################################################################################
    Public Function ToBitmap() As Bitmap
        Dim I As Integer
        Dim J As Integer
        Dim bImg As New Bitmap(Width, Height)

        'loop through every pixel in this image
        For I = 0 To Width - 1
            For J = 0 To Height - 1

                'save each pixel in its component format
                bImg.SetPixel(I, J, Color.FromArgb(R(I, J), G(I, J), B(I, J)))
            Next J
        Next I

        'return the completed image.
        Return bImg

    End Function


End Class
