Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' TimeStamp: a timer container allowing precision at long intervals. This class allows the 
' user to start a timer and allow it to run for long periods of time, yet still maintain 
' fraction-of-a-second precision (55ms in Windows 9x, 10ms in 2000/XP)
'
' Interfaces Required:  iTestable
' Classes Required:     ConstraintCollection
' Modules Required:     none
' Other Requirements:   none
'
' Certification:        In Progress (05/23/04)
'                       Module Active in 0 Projects
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (12/01/03)    Reviewed code and comments for errors and ommissions
'
' (12/01/03)    Tested the file for 15 minutes, set the file to read-only and started the 
'               code certification process.
'
' (02/18/04)    Reformatted comment bars and comments.
'
' (03/12/04)    MILESTONE: class shipped with project ASC2288.
'
' (05/12/04)    MILESTONE: class shipped with project ASC DrawMove (internal).
'
' (05/23/04)    Removed the old-style Test() function, to be replaced with a new version of
'               the function.
'
'#############################################################################################

Namespace naDvc

    Public Class TimeStamp

        'define container for holding the exact date and time this timer was started
        Private tmStart As New DateTime(0)
        Private bmStarted As Boolean = False


        '#####################################################################################
        'return the exact amount of time elapsed since inception
        '#####################################################################################
        Public ReadOnly Property Elapsed() As TimeSpan

            Get
                If bmStarted Then
                    'return actual time passed if the timer has started
                    Elapsed = Now.Subtract(tmStart)
                Else
                    'timer didn't start yet? then return exactly zero
                    Elapsed = New TimeSpan(0)
                End If
            End Get

        End Property


        '#####################################################################################
        'start execution of the timer
        '#####################################################################################
        Public Sub Start()
            tmStart = Now
            bmStarted = True
        End Sub

    End Class

End Namespace