Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' YUVImage: a container for an array of RGB colors.
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (02/08/05)    Initial creation of the class.
'
'#############################################################################################

Public Class YUVImage
    Inherits ColorImage

    Public Y(,) As Integer
    Public U(,) As Integer
    Public V(,) As Integer


    '#########################################################################################
    'create a new image. all color arrays have the same size and depth
    '#########################################################################################
    Sub New(ByVal iWidth As Integer, ByVal iHeight As Integer)
        MyBase.New(iWidth, iHeight)
    End Sub


    '#########################################################################################
    'resize the image. data is never preserved during a resize.
    '#########################################################################################
    Protected Overrides Sub Resize()

        ReDim Y(imWidth - 1, imHeight - 1)
        ReDim U(imWidth - 1, imHeight - 1)
        ReDim V(imWidth - 1, imHeight - 1)

    End Sub


    '#########################################################################################
    'loop through all members of this array and convert it to an RGB image.
    '#########################################################################################
    Public Function ToRGB() As RGBImage
        Dim mRGB As RGBImage
        Dim I As Integer
        Dim J As Integer

        'create the new RGB image to return
        mRGB = New RGBImage(Width, Height)

        'loop through every pixel in this image
        For I = 0 To Width - 1
            For J = 0 To Height - 1

                'convert every pixel to the correct RGB formatYUV
                mRGB.R(I, J) = Constrain(Me.Y(I, J) + (1.4075 * (Me.V(I, J) - 128)))
                mRGB.G(I, J) = Constrain(Me.Y(I, J) - (0.3455 * (Me.U(I, J) - 128)) - (0.7169 * (Me.V(I, J) - 128)))
                mRGB.B(I, J) = Constrain(Me.Y(I, J) + (1.779 * (Me.U(I, J) - 128)))

            Next J
        Next I

        'return the result
        Return mRGB

    End Function


End Class
