Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' iHandler: an interface that, once implemented, guarantees that a class is able to report
' errors. A class that implements this interface can be either stateful or stateless: stateful
' implementations keep track of errExists, display a certain condition when an error exists, 
' and clear that condition once the error is cleared. Stateless implementations simply report
' errors and do not attempt to keep track of whether an error exists or not.
'
' Interfaces Required:  State
' Classes Required:     none
' Modules Required:     none 
' Other Requirements:   none
'
' Certification:        In Progress (05/19/04)
'                       Module Active in 0 Projects
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (08/13/03)    Reviewed code and comments for errors and ommissions.
'
' (08/13/03)    Tested the file for 5 minutes, set the file to read-only and started the code 
'               certification process.
'
' (02/11/04)    Reformatted comment bars and comments.
'
' (03/12/04)    MILESTONE: class shipped with project ASC2288.
'
' (05/06/04)    Began a major rewrite to convert the class from an error-based two-state
'               class to a generic n-state class. The resulting changes simplified the
'               class greatly. Since this is just an interface, its testing is more 
'               theoretical and dependent on its application.
'
' (05/12/04)    MILESTONE: class shipped with project ASC DrawMove (internal).
'
' (05/19/04)    To conform with the newest error handling system, made the State property
'               read-only. Ran the interface with a few different classes to confirm that
'               it's the right path to go.
'
'#############################################################################################

Namespace naExc

    Public Interface iHandler
        'what's our current state? read the current state or assign a new state
        ReadOnly Property State() As naExc.State

        'output a simple line of debug text without altering state
        Sub debugText(ByVal sText As String)
    End Interface

End Namespace