Option Explicit On 
Option Strict On

'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' AVICapControl: a picturebox-derived control designed to display a web or USB camera's
' real-time video feed. This control uses the AVICap drivers (Video for Windows), so it should
' be fairly robust and flexible.
'
' References:
' http://www.codeproject.com/vb/net/BitBlt.asp
'
' Interfaces Required:  iGenerator
' Classes Required:     none
' Modules Required:     none 
' Other Requirements:   none
'
' Certification:        NOT In Progress (05/xx/04)
'                       Module Active in 0 Project
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (04/19/04)    Started work on the control.
'
'#############################################################################################

Imports System.Runtime.InteropServices
Imports ASCCLS.naExc

<ToolboxBitmap(GetType(ColorDialog))> Public Class AVICapControl
    Inherits System.Windows.Forms.UserControl
    Implements ASCCLS.naExc.iGenerator

    'declare two critical delegate functions, similar to events
    Private Delegate Function cameraStatusDelegate(ByVal iHWnd As Integer, ByVal iID As Integer, ByVal ipstrStatusText As Integer) As Integer
    Private Delegate Function cameraErrorDelegate(ByVal iHWnd As Integer, ByVal iID As Integer, ByVal ipstrStatusText As Integer) As Integer

    'declare two critical functions for capturing video
    Private Declare Function capCreateCaptureWindowA Lib "avicap32.dll" (ByVal sWindowName As String, ByVal iStyle As Integer, ByVal iX As Integer, ByVal iY As Integer, ByVal iWidth As Integer, ByVal iHeight As Short, ByVal hWndParent As Integer, ByVal iID As Integer) As Integer
    Private Declare Function capGetDriverDescriptionA Lib "avicap32.dll" (ByVal iDriver As Short, ByVal sName As String, ByVal iName As Integer, ByVal sVer As String, ByVal iVer As Integer) As Boolean

    'functions for modifying and accessing the API directly. three overloads are necessary
    Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal iHWnd As Integer, ByVal iMsg As Integer, ByVal iParam As Integer, ByVal lParam As Object) As Integer
    Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal iHWnd As Integer, ByVal iMsg As Integer, ByVal wParam As Short, ByVal oProcD As cameraStatusDelegate) As Integer
    Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal iHWnd As Integer, ByVal iMsg As Integer, ByVal wParam As Short, ByVal oProcD As cameraErrorDelegate) As Integer

    'functions for initializing and destroying a window
    Private Declare Function SetWindowPos Lib "user32" Alias "SetWindowPos" (ByVal iHWnd As Integer, ByVal iWndInsertAfter As Integer, ByVal iX As Integer, ByVal iY As Integer, ByVal iCX As Integer, ByVal iCY As Integer, ByVal iFlags As Integer) As Integer
    Private Declare Function DestroyWindow Lib "user32" (ByVal iHWnd As Integer) As Boolean
    Private Declare Function lStrCpy Lib "kernel32" Alias "lstrcpyA" (ByVal lpString1 As Integer, ByVal lpString2 As Integer) As Integer
    Private Declare Function lStrCpy Lib "kernel32" Alias "lstrcpyA" (ByVal lpString1 As String, ByVal lpString2 As String) As Integer

    'graphics routines for capturing the image as a single shot
    Private Declare Auto Function BitBlt Lib "gdi32.dll" (ByVal hdcDest As IntPtr, ByVal nXDest As Integer, ByVal nYDest As Integer, ByVal nWidth As Integer, ByVal nHeight As Integer, ByVal hdcSrc As IntPtr, ByVal nXSrc As Integer, ByVal nYSrc As Integer, ByVal dwRop As Int32) As Boolean
    Private Declare Auto Function GetPixel Lib "gdi32" (ByVal hDC As IntPtr, ByVal iX As Integer, ByVal iY As Integer) As Integer

    Private Const WS_CHILD As Integer = &H40000000
    Private Const WS_VISIBLE As Integer = &H10000000

    Private Const WM_CAP As Short = &H400S
    Private Const WM_CAP_DRIVER_CONNECT As Integer = WM_CAP + 10
    Private Const WM_CAP_DRIVER_DISCONNECT As Integer = WM_CAP + 11
    Private Const WM_CAP_DRIVER_GET_NAME As Integer = WM_CAP + 12
    Private Const WM_CAP_DRIVER_GET_VERSION As Integer = WM_CAP + 13
    Private Const WM_CAP_DRIVER_GET_CAPS As Integer = WM_CAP + 14
    Private Const WM_CAP_SET_CALLBACK_ERROR As Integer = WM_CAP + 2
    Private Const WM_CAP_SET_CALLBACK_STATUS As Integer = WM_CAP + 3

    Private Const WM_CAP_EDIT_COPY As Integer = WM_CAP + 30
    Private Const WM_CAP_SET_PREVIEW As Integer = WM_CAP + 50
    Private Const WM_CAP_SET_PREVIEWRATE As Integer = WM_CAP + 52
    Private Const WM_CAP_SET_SCALE As Integer = WM_CAP + 53

    Private Const WM_CAP_DLG_VIDEOFORMAT As Integer = WM_CAP + 41
    Private Const WM_CAP_DLG_VIDEOSOURCE As Integer = WM_CAP + 42
    Private Const WM_CAP_DLG_VIDEOCOMPRESSION As Integer = WM_CAP + 46

    'setwindowpos constants
    Private Const HWND_TOP As Integer = 0
    Private Const HWND_BOTTOM As Integer = 1
    Private Const HWND_NOTOPMOST As Integer = -2
    Private Const HWND_TOPMOST As Integer = -1
    Private Const SWP_FRAMECHANGED As Integer = &H20
    Private Const SWP_DRAWFRAME As Integer = SWP_FRAMECHANGED
    Private Const SWP_HIDEWINDOW As Integer = &H80
    Private Const SWP_NOACTIVATE As Integer = &H10
    Private Const SWP_NOCOPYBITS As Integer = &H100
    Private Const SWP_NOMOVE As Integer = &H2
    Private Const SWP_NOOWNERZORDER As Integer = &H200
    Private Const SWP_NOREDRAW As Integer = &H8
    Private Const SWP_NOREPOSITION As Integer = SWP_NOOWNERZORDER
    Private Const SWP_NOSIZE As Integer = &H1
    Private Const SWP_NOZORDER As Integer = &H4
    Private Const SWP_SHOWWINDOW As Integer = &H40

    'status string ID for a live window
    Private Const IDS_CAP_STAT_LIVE_MODE As Integer = 500

    'bitblt constant
    Private Const SRCCOPY As Integer = &HCC0020

    'my own important constant: this is the sum of all color parts of the "visible"
    'bitmap (the camera) for verifying visibility.
    Private Const CAMERA_CHECKSUM As Integer = 18635

    'now we can define the actual private members of this class.
    Private omHandlers As New ASCCLS.naExc.HandlerCollection
    Private smDevice As String = ""

    Private igXScale As Integer = 640
    Private igYScale As Integer = 480
    Private igXOffset As Integer = 0
    Private igYOffset As Integer = 0

    Private imHWnd As Integer
    Private bmAutoScale As Boolean = True

    Private lmImgChecksum As Long = 0
    Private imImgChecks As Integer = 0
    Private smID As String = "AVI Capture Control"


#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()
        Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.Olive, "Device is initializing...")

        'Add any initialization after the InitializeComponent() call

    End Sub

    'UserControl overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents tmrActive As System.Timers.Timer
    Friend WithEvents pctAVI As System.Windows.Forms.PictureBox
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.tmrActive = New System.Timers.Timer
        Me.pctAVI = New System.Windows.Forms.PictureBox
        CType(Me.tmrActive, System.ComponentModel.ISupportInitialize).BeginInit()
        Me.SuspendLayout()
        '
        'tmrActive
        '
        Me.tmrActive.Interval = 1000
        Me.tmrActive.SynchronizingObject = Me
        '
        'pctAVI
        '
        Me.pctAVI.Dock = System.Windows.Forms.DockStyle.Fill
        Me.pctAVI.Location = New System.Drawing.Point(0, 0)
        Me.pctAVI.Name = "pctAVI"
        Me.pctAVI.Size = New System.Drawing.Size(416, 264)
        Me.pctAVI.TabIndex = 2
        Me.pctAVI.TabStop = False
        '
        'AVICapControl
        '
        Me.Controls.Add(Me.pctAVI)
        Me.Name = "AVICapControl"
        Me.Size = New System.Drawing.Size(416, 264)
        CType(Me.tmrActive, System.ComponentModel.ISupportInitialize).EndInit()
        Me.ResumeLayout(False)

    End Sub
#End Region


    '#####################################################################################
    'set/get the id used to reference (and raise errors) for this class uniquely
    '#####################################################################################
    Public Property ID() As String Implements ASCCLS.naExc.iGenerator.ID

        Get
            Return smID
        End Get

        Set(ByVal Value As String)
            smID = Value
        End Set

    End Property


    '#########################################################################################
    'retrieve the error handlers collection to satisfy the iGenerator interface and allow 
    'exception handling
    '#########################################################################################
    Public ReadOnly Property Handlers() As ASCCLS.naExc.HandlerCollection Implements ASCCLS.naExc.iGenerator.Handlers

        Get
            Handlers = omHandlers
        End Get

    End Property


    '#########################################################################################
    'return a collection of strings representing each driver available for video capture
    'a dictionary is returned, where the device's name is the key and its number is the
    'value
    '#########################################################################################
    Public Function Devices() As Hashtable

        'initialize two strings for passing to a strict API
        Dim sName As String = Space(255)
        Dim sVer As String = Space(255)
        Dim sAdd As String
        Dim bReturn As Boolean
        Dim I As Integer = 0
        Dim J As Integer

        Dim oDrivers As New Hashtable

        Try
            'loop through all returnable drivers until no driver is returned.
            Do
                bReturn = capGetDriverDescriptionA(CShort(I), sName, 255, sVer, 255)
                If bReturn Then
                    'the API function returns a zero-ASCII-terminated string. find the first
                    'zero and return everything before that.
                    For J = 1 To sName.Length
                        If Asc(Mid(sName, J, 1)) = 0 Then Exit For
                    Next J

                    'parse out all zero characters
                    sAdd = Mid(sName, 1, J - 1)
                    oDrivers.Add(sAdd.Trim, I)
                End If

                I += 1
            Loop Until bReturn = False

            'finally, return the driver list
            Return oDrivers

        Catch eX As Exception
            'if no drivers are available or there is any other type of problem, immediately
            'return and handle an exception. return an empty collection
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Error initializing video capture devices: " & eX.Message)
            Return New Hashtable

        End Try

    End Function


    '#########################################################################################
    'get the device being used
    '#########################################################################################
    Public ReadOnly Property Device() As String
        Get
            Return smDevice
        End Get
    End Property


    '#########################################################################################
    'within the Video for Windows generic driver architecture, the specific drivers are
    'expected to provide a dialog box for altering that device's specific video format.
    'this call allows this control to access its dialog.
    '#########################################################################################
    Public Sub showVideoFormatDialog()

        If Me.Active = False Then
            'if no picture is active, then there's no reason to display.
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot show video format dialog for an inactive video capture device.")
        Else
            'everything's cool. display it.
            SendMessage(imHWnd, WM_CAP_DLG_VIDEOFORMAT, 0, 0)
        End If

    End Sub


    '#########################################################################################
    'scan two diagonal lines across the viewing window and record the sum of all colors (the
    'sum of all reds, greens, and blues). this method is best used as a check to see if a
    'connection was lost to a camera. a frozen picture will return identical signatures (since
    'the picture never changes) over a few tries, while a non-frozen picture should change
    'all the time (minutely occasionally, but still changing)
    '
    'not only does this function confirm that the picture is dynamic (changing), but since
    'it captures the screen which is visible, it confirms that the picture is being shown and
    'will be saved correctly.
    '#########################################################################################
    Private Function scanPicture(ByVal bCur As Bitmap) As Long
        Dim iPoints As Integer
        Dim I As Integer

        Dim iX As Integer
        Dim iY As Integer

        Dim cPix As Color

        'our sum could be huge. 64 bits of integer should be sufficient
        Dim lSum As Long = 0

        'now, cut a simple pattern across the picture. the number of points total for each
        'cut will be equal to the longest side (width or height)
        If bCur.Width > bCur.Height Then iPoints = bCur.Width Else iPoints = bCur.Height

        'start scanning through points. this scans a line diagonally from the top left of
        'the picture to the bottom right.
        lSum = 0
        For I = 0 To iPoints - 1
            'get the current pixel
            iX = CInt(Math.Floor((I / iPoints) * bCur.Width))
            iY = CInt(Math.Floor((I / iPoints) * bCur.Height))

            'now, get this pixel's color value
            cPix = bCur.GetPixel(iX, iY)

            'add each RGB component to the color sum
            lSum += cPix.R
            lSum += cPix.G
            lSum += cPix.B
        Next I

        'we've completed the diagonal scan.
        Return lSum

    End Function


    '#########################################################################################
    'capture the currently-displayed graphics within this control's bounds into a bitmap.
    'most of this code was referenced from an external website (see references in top section)
    '#########################################################################################
    Public Function ToBitmap() As Bitmap
        Dim iX As Integer = MyBase.Width
        Dim iY As Integer = MyBase.Height

        'get a graphics object from this control
        Dim gPic As Graphics = MyBase.CreateGraphics

        'create an empty bitmap from that graphics
        Dim bImg As Bitmap = New Bitmap(iX, iY, gPic)

        'create a graphics object in memory from that bitmap
        Dim gBit As Graphics = Graphics.FromImage(bImg)

        'get the IntPtr's of the graphics
        Dim oHPic As IntPtr = gPic.GetHdc
        Dim oHBit As IntPtr = gBit.GetHdc

        'get the picture using bitblt now
        BitBlt(oHBit, 0, 0, iX, iY, oHPic, 0, 0, 13369376)

        'clone the bitmap so we can dispose this one 
        ToBitmap = CType(bImg.Clone(), Bitmap)
        bImg.Dispose()

        'clean up all our loose ends 
        gPic.ReleaseHdc(oHPic)
        gBit.ReleaseHdc(oHBit)
        gPic.Dispose()
        gBit.Dispose()

    End Function


    '#########################################################################################
    'same general idea as above; different dialog type
    '#########################################################################################
    Public Sub showVideoSourceDialog()

        If Me.Active = False Then
            'if no picture is active, then there's no reason to display.
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot show video source dialog for an inactive video capture device.")
        Else
            'everything's cool. display it.
            SendMessage(imHWnd, WM_CAP_DLG_VIDEOSOURCE, 0, 0)
        End If

    End Sub


    '#########################################################################################
    'again, same idea; this form allows the user to choose a video codec for compression
    'not sure how useful this is. it may just be for recording video.
    '#########################################################################################
    Public Sub showVideoCompressionDialog()

        If Me.Active = False Then
            'if no picture is active, then there's no reason to display.
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot show video compression dialog for an inactive video capture device.")
        Else
            'everything's cool. display it.
            SendMessage(imHWnd, WM_CAP_DLG_VIDEOCOMPRESSION, 0, 0)
        End If

    End Sub


    '#########################################################################################
    'return or set the single driver used for this capture window. the driver must be
    'validated in the Drivers() list.
    '#########################################################################################
    Public Sub setDevice(ByVal sValue As String)

        'if we have a connection currently open, can't change driver.
        If Me.Active Then
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot change video capture device while video capture is active.")
        Else
            'loop through all drivers. only assign a driver if it exists in the list
            If Me.Devices.Contains(sValue) Then
                smDevice = sValue
            Else
                'if our selection is never assigned, raise some flak
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "The video capture device [" & sValue & "] does not exist.")
            End If
        End If

    End Sub


    '#########################################################################################
    'is the capture window capturing now? return that value
    '#########################################################################################
    Public ReadOnly Property Active() As Boolean

        Get
            Return tmrActive.Enabled
        End Get

    End Property


    '#########################################################################################
    'how often should the software automatically (and independently of the camera drivers and
    'API) verify that the camera image is dynamic and changing? like a timer, an interval of
    'zero prevents it from firing.
    '#########################################################################################
    Public Property verInterval() As Double

        Get
            Return tmrActive.Interval
        End Get

        Set(ByVal Value As Double)
            tmrActive.Interval = Value
        End Set

    End Property


    '#########################################################################################
    'determine the x and y dimensions of the camera image. these values can be used to
    'scale the camera image and for cropping out unneeded portions of the image.
    '#########################################################################################
    Public Property xScale() As Integer

        Get
            If Me.autoScale Then Return Me.Width Else Return igXScale
        End Get

        Set(ByVal Value As Integer)

            'if we have a connection currently open, can't change it. we can set its value 
            'even if autoScale is True; it just won't be used until autoScale is False.
            'if
            If Me.Active Then
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot change property while video capture is active.")
            Else
                igXScale = Value
            End If

        End Set

    End Property


    '#########################################################################################
    'same idea as above, but height rather than width
    '#########################################################################################
    Public Property yScale() As Integer

        Get
            If Me.autoScale Then Return Me.Height Else Return igYScale
        End Get

        Set(ByVal Value As Integer)

            'if we have a connection currently open, can't change it. we can set its value 
            'even if autoScale is True; it just won't be used until autoScale is False.
            'if
            If Me.Active Then
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot change property while video capture is active.")
            Else
                igYScale = Value
            End If

        End Set

    End Property


    '#########################################################################################
    'the user can also offset the centerpoint of the image to adjust how the image is
    'cropped.
    '#########################################################################################
    Public Property xOffset() As Integer

        Get
            If Me.autoScale Then Return 0 Else Return igXOffset
        End Get

        Set(ByVal Value As Integer)

            'if we have a connection currently open, can't change it. we can set its value 
            'even if autoScale is True; it just won't be used until autoScale is False.
            If Me.Active Then
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot change property while video capture is active.")
            Else
                igXOffset = Value
            End If

        End Set

    End Property


    '#########################################################################################
    'same idea as above; alter the y (height) offset
    '#########################################################################################
    Public Property yOffset() As Integer

        Get
            If Me.autoScale Then Return 0 Else Return igYOffset
        End Get

        Set(ByVal Value As Integer)

            'if we have a connection currently open, can't change it. we can set its value 
            'even if autoScale is True; it just won't be used until autoScale is False.
            If Me.Active Then
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot change property while video capture is active.")
            Else
                igYOffset = Value
            End If

        End Set

    End Property


    '#########################################################################################
    'should the video picture be scaled automatically to this control's width and height
    'upon connection? or should manual gain and offset values be used?
    '#########################################################################################
    Public Property autoScale() As Boolean

        Get
            Return bmAutoScale
        End Get

        Set(ByVal Value As Boolean)

            'if we have a connection currently open, can't change it
            If Me.Active Then
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Cannot change property while video capture is active.")
            Else
                bmAutoScale = Value
            End If

        End Set

    End Property


    '#########################################################################################
    'connect this display window to a video source using the previously assigned driver
    'reference. supplying custom widths and heights allows us to create a window that is
    'not identical in size
    '#########################################################################################
    Public Sub Connect()
        Dim iDevice As Integer
        Dim bStatus As Boolean

        Dim iLeft As Integer
        Dim iTop As Integer

        Try
            'retrieve the device number of the currently selected device
            iDevice = CInt(Me.Devices(smDevice))

            'set the window position to overlap our frame's position. then, calculate
            'the correct top-left location for the frame utilizing the image's 
            'dimensions, the control's dimensions, and the assigned offsets
            iLeft = (Me.Width - Me.xScale) \ 2 + Me.xOffset
            iTop = (Me.Height - Me.yScale) \ 2 + Me.yOffset

            'set up the capture window and return the capture's handle for later work
            'the 640x480 resolution specified here does not appear to affect actual
            'scale in any way
            imHWnd = capCreateCaptureWindowA(Me.Text, WS_VISIBLE Or WS_CHILD, iLeft, iTop, _
                                            640, 480, pctAVI.Handle.ToInt32, 0)

            'set up callback functions (very similar to events, basically) that the
            'API can raise in regards to status messages and errors
            SendMessage(imHWnd, WM_CAP_SET_CALLBACK_STATUS, CShort(0), CType(AddressOf cameraStatus, cameraStatusDelegate))
            SendMessage(imHWnd, WM_CAP_SET_CALLBACK_ERROR, CShort(0), CType(AddressOf cameraError, cameraErrorDelegate))

            'now explicitly connect the device to the specified hWnd using a message send
            bStatus = CBool(SendMessage(imHWnd, WM_CAP_DRIVER_CONNECT, iDevice, 0))

            'confirm that the connection was successful
            If bStatus Then
                'set up the capture device's scaling properties and preview properties
                SendMessage(imHWnd, WM_CAP_SET_SCALE, 1, 0)
                SendMessage(imHWnd, WM_CAP_SET_PREVIEWRATE, 30, 0)
                SendMessage(imHWnd, WM_CAP_SET_PREVIEW, 1, 0)
                SendMessage(imHWnd, WM_CAP_SET_SCALE, 1, 0)

                'set up flags and messages indicating we're good to go
                tmrActive.Enabled = True
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Nominal, Color.DarkGreen, "Camera communications active.")

                'finally, set the frame's position onto the control
                SetWindowPos(imHWnd, HWND_BOTTOM, 0, 0, Me.xScale, Me.yScale, SWP_NOMOVE)
            Else
                'connection was unsuccessful? in that case, destroy our hWnd capture window
                'and raise an error specifying that we couldn't make the connection.
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Attempt to open a connection to video capture device [" & smDevice & "] failed.")
                DestroyWindow(imHWnd)
            End If

        Catch eX As Exception
            'if no drivers are available or there is any other type of problem, immediately
            'return and handle an exception. return an empty collection
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Error attempting to open a connection to video capture device [" & smDevice & "]: " & eX.Message)
        End Try

    End Sub


    '#########################################################################################
    'disconnect from the AVI capture device and destroy its window
    '#########################################################################################
    Public Sub Disconnect()
        Dim iDevice As Integer

        Try
            'retrieve the device number of the currently selected device
            iDevice = CInt(Me.Devices(smDevice))

            'send a message to disconnect the capture driver explicitly, and then
            'destroy the associated capture device handle
            SendMessage(imHWnd, WM_CAP_DRIVER_DISCONNECT, iDevice, 0)
            DestroyWindow(imHWnd)
            tmrActive.Enabled = False
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "The vision window has been software disconnected from the camera.")

        Catch eX As Exception
            'if no drivers are available or there is any other type of problem, immediately
            'return and handle an exception. return an empty collection
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Error attempting to close a connection to video capture device [" & smDevice & "]: " & eX.Message)
        End Try

    End Sub


    '#########################################################################################
    'a delegated callback function (similar to an event) raised by the API when the
    'camera's status changes. status messages are typically a good thing, raised when
    'the camera starts talking to us again
    '#########################################################################################
    Private Function cameraStatus(ByVal iHWnd As Integer, ByVal iID As Integer, ByVal ipstrStatusText As Integer) As Integer
        Dim sMsg As String

        'a message identifier of zero indicates a new operation is starting and the 
        'callback function should clear the current status
        If iID = 0 Then Exit Function

        'using the marshalling process, grab the string that the pointer points to
        sMsg = Marshal.PtrToStringAnsi(New IntPtr(ipstrStatusText))

        'if we just received a message that the camera is displaying a live mode window,
        'then automatically clear any errors.
        If iID = IDS_CAP_STAT_LIVE_MODE Then
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Nominal, Color.DarkGreen, sMsg)
        Else
            Me.Handlers.debugText("Status message " & iID & ": " & sMsg)
        End If

    End Function


    '#########################################################################################
    'a delegated callback function (similar to an event) raised by the API when some
    'sort of error occurs with the camera.
    '#########################################################################################
    Private Function cameraError(ByVal iHWnd As Integer, ByVal iID As Integer, ByVal ipstrStatusText As Integer) As Integer
        Dim sMsg As String

        'using the marshalling process, grab the string that the pointer points to
        sMsg = Marshal.PtrToStringAnsi(New IntPtr(ipstrStatusText))

        'errors are always a bad thing. raise the error.
        Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Error " & iID & ": " & sMsg)

    End Function


    '#########################################################################################
    'the user's rescaling the capture window. keep up with him.
    '#########################################################################################
    Protected Overloads Overrides Sub OnResize(ByVal eX As System.EventArgs)

        'always resize the base window before proceeding
        MyBase.OnResize(eX)

        'now, if we're in automatic scaling mode, rescale the image to match dimensions
        If Me.Active And Me.autoScale Then
            'finally, set the frame's position onto the control
            SetWindowPos(imHWnd, HWND_BOTTOM, 0, 0, Me.Width, Me.Height, SWP_NOMOVE Or SWP_NOZORDER)
        End If

    End Sub


    '#########################################################################################
    'check to see if the image displayed is the same as the last time this timer ticked off.
    '#########################################################################################
    Private Sub tmrActive_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles tmrActive.Elapsed
        Dim lCur As Long = 0
        Dim sTime As String
        Dim cPix As Color
        Dim I As Integer

        'immediately retrieve the current bitmap
        Dim bCur As Bitmap = Me.ToBitmap()

        'if the window is too small, then checking is impossible
        If bCur.Width < 32 Or bCur.Height < 32 Then
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "Camera image window must have a width and height creater than 32 pixels.")
            bCur.Dispose()
            Exit Sub
        End If

        'first, check the cross-section of the first 32 diagonal area bits. this will
        'verify that the picture area is physically visible.
        For I = 0 To 31
            cPix = bCur.GetPixel(I, I)
            lCur += cPix.R
            lCur += cPix.G
            lCur += cPix.B
        Next I

        'if our camera checksum does not match, then the user hid the camera form (and
        'any picture we try to record will be invalid). tell the user he's a moron.
        If lCur <> CAMERA_CHECKSUM Then
            Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "The camera image is not visible on the screen. To store images, the camera image must be visible.")
            bCur.Dispose()
            Exit Sub
        End If

        'now, call the somewhat-external function to retrieve the diagonal checksum
        'of the currently-displayed image. then, destroy the bitmap.
        lCur = scanPicture(bCur)
        bCur.Dispose()

        If lCur = lmImgChecksum Then
            'if the image signature currently being displayed is not the same as the one
            'we last pulled, increase the warning counter. if the warning counter exceeds
            'four, then raise an error
            imImgChecks += 1

            'raise an error if we exceed value.
            If imImgChecks > 3 Then
                sTime = Format(CDbl(tmrActive.Interval * imImgChecks) / 1000, "0.0")
                Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Fault, Color.DarkRed, "The visible camera image has been static for " & sTime & " seconds. This may be caused by a malfunctioning camera.")
            End If
        Else
            'the image signature is different. reset the warning counter and assign the
            'image signature to the backed up one
            imImgChecks = 0
            lmImgChecksum = lCur
            If Me.Active Then Me.Handlers.State.Change(Me.ID, ASCCLS.naExc.State.Nominal, Color.DarkGreen, "Camera is active and transmitting images.")
        End If

    End Sub

    Private Sub pctCamera_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        If Me.Active Then Disconnect() Else Connect()
    End Sub
End Class
