Option Explicit On 
Option Strict On


'################################################################################
' John Girard, Advanced Systems & Controls
'
' HistogramFilter: takes as an input an array of numbers, and returns a
' histogram as its output. the histogram starts at the array's minimum value,
' and ends at its maximum value. the histogram contains a specified number
' of bins as well, which may be changed.
'
' Interfaces Required:  none
' Classes Required:     Filter
' Modules Required:     ASCMath
' Other Requirements:   none
'
' Certification:        NOT In Progress (01/xx/04)
'                       Module Active in 0 Projects
'
' Revision History:
' ------------------------------------------------------------------------------
'
' (12/29/03)    Initial creation of the class.
'
' (01/xx/04)    Tested the file for xx minutes, set the file to read-only 
'               and started the code certification process.
'
'requires 45 minutes testing
'################################################################################
Namespace naFilter

    Public Class HistogramFilter
        Inherits naFilter.Filter

        Private imBins As Integer = 0
        Private dmBinWidth As Double = 0


        '########################################################################
        'return the number of bins in this histogram. in other words, return
        'the number of bars in the histogram bar graph, the resolution of the
        'histogram.
        '########################################################################
        Public Property Bins() As Integer

            Get
                Bins = imBins
            End Get

            Set(ByVal Value As Integer)
                imBins = Value
            End Set

        End Property


        '########################################################################
        'return the width of each bin. only valid once the data has been filtered
        '########################################################################
        Public Property BinWidth() As Double

            Get
                BinWidth = dmBinWidth
            End Get

            Set(ByVal Value As Double)
                dmBinWidth = Value
            End Set

        End Property


        '########################################################################
        'execute the filter for the histogram
        '########################################################################
        Public Overrides Function Filter() As Double
            Dim dDest() As Double
            Dim dSamples As Double
            Dim dR As Double
            Dim iBin As Integer

            Dim I As Integer

            'create the resultant data with exactly the right number of bins
            ReDim dDest(Bins - 1)

            'using the range of the array to be filtered, determine how wide
            'each bin is
            dmBinWidth = asRange(MyBase.dmSrc) / Bins

            'now, loop through each item in the source array and add each of
            'these points to its corresponding bin
            For I = 0 To MyBase.dmSrc.Length - 1

                'figure out which bin this point is in
                iBin = CInt(MyBase.dmSrc(I) / dmBinWidth)
                If iBin > Bins - 1 Then iBin = Bins - 1
                If iBin < 0 Then iBin = 0

                dDest(iBin) += 1
            Next I

            'determine the compression ratio
            dSamples = Bins / MyBase.dmSrc.Length

            'assign the array and return compression ratio, or change in sample space
            MyBase.dmSrc = dDest
            Return dSamples

        End Function


        '##################################################################
        'create the base new declaration
        '##################################################################
        Public Sub New()
            MyBase.New()
        End Sub


        '##################################################################
        'initialize with new number of bins
        '##################################################################
        Public Sub New(ByVal iBins As Integer)
            MyBase.New()
            Bins = iBins
        End Sub

    End Class

End Namespace
