Module deadCode

    '#############################################################################################
    'construct a segmentation map of the passed-in, filtered chrominances. the input array
    'consists of integers relating to a single color match. values set to zero represent no
    'match, while other values represent a single color match. only like colors are regioned
    'together.
    '#############################################################################################
    Public Function Regionize(ByVal iMaxColor As Integer, ByVal iColor(,) As Integer) As Integer(,)
        Dim I As Integer
        Dim J As Integer
        Dim K As Integer

        Dim iRegion(,) As Integer
        Dim iTemp() As Integer
        Dim iLabels() As Integer
        Dim bNeighbor As Boolean
        Dim iTotalRegions As Integer = 0

        'the collection of collision locations
        Dim oCollisions As New Collection

        'create an array that can serve as a reference for the labeling process
        ReDim iRegion(iColor.GetUpperBound(0), iColor.GetUpperBound(1))
        ReDim iLabels(iMaxColor - 1)

        'loop through every single pixel in the map
        For I = 0 To iColor.GetLength(0) - 1
            For J = 0 To iColor.GetLength(1) - 1
                bNeighbor = False

                'does the current pixel have a color (any valid color)?
                If iColor(I, J) > 0 Then

                    'generate an array of all neighbors of this current pixel
                    iTemp = getNeighbors(iRegion, I, J)

                    'loop through all neighbors. try to find one that already
                    'has the current label assigned to it. colors are assigned
                    'a repeating pattern of integers.
                    For K = 0 To iTemp.Length - 1
                        If iTemp(K) = iColor(I, J) Mod iMaxColor Then
                            'choose the current neighbor and use its value for
                            'the current label.
                            iRegion(I, J) = iTemp(K)
                            bNeighbor = True
                        End If
                    Next

                    'all neighbors have mis-matching colors or no colors at
                    'all. create a brand-new label for this one.
                    If K = iTemp.Length Then
                        iTotalRegions += 1
                        iLabels(iColor(I, J) - 1) += 1
                        iRegion(I, J) = iColor(I, J) * iLabels(iColor(I, J) - 1)
                    End If

                    'was a neighbor used as the source of a label? if so, then
                    'check all the other neighbors and see if they either match
                    'the current color (but with a different mod value) or have 
                    'no label at all.
                    If bNeighbor Then

                        'loop through all neighbors, looking for a non-match
                        For K = 0 To iTemp.Length - 1
                            If iTemp(K) <> iRegion(I, J) And iTemp(K) > 0 Then
                                'does the current value match the correct color,
                                'only with a different modulus?
                                If iTemp(K) Mod iMaxColor = iRegion(I, J) Mod iMaxColor Then
                                    'if there's a collision of colors, then we
                                    'have to add it to the combination list
                                    oCollisions.Add(New Point(iTemp(K), iRegion(I, J)))
                                End If
                            End If
                        Next
                    End If
                End If
            Next
        Next

        Return iRegion

    End Function


    '#############################################################################################
    'return all the neighbor values of a single point. between 2 and 4 neighbors will be returned.
    '#############################################################################################
    Public Function getNeighbors(ByVal iArray(,) As Integer, ByVal iX As Integer, ByVal iY As Integer) As Integer()
        Dim iNeighbors() As Integer
        Dim iCount As Integer = 0

        'assume all neighbors exist
        ReDim iNeighbors(3)

        'is there a neighbor to the west?
        If iX > 0 Then
            iNeighbors(iCount) = iArray(iX - 1, iY)
            iCount += 1
        End If

        'is there a neighbor to the east?
        If iX < iArray.GetLength(0) - 1 Then
            iNeighbors(iCount) = iArray(iX + 1, iY)
            iCount += 1
        End If

        'is there a neighbor to the north?
        If iY > 0 Then
            iNeighbors(iCount) = iArray(iX, iY - 1)
            iCount += 1
        End If

        'is there a neighbor to the south?
        If iY < iArray.GetLength(1) - 1 Then
            iNeighbors(iCount) = iArray(iX, iY + 1)
            iCount += 1
        End If

        'redimension the number of neighbors to match
        ReDim Preserve iNeighbors(iCount)
        Return iNeighbors

    End Function

End Module
