Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' FuzzyColor: this is a fuzzy representation of a color which can be segmented. The color
' is stored in YUV format
'
' References:
' http://mathworld.wolfram.com/NormalDistribution.html
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (01/25/05)    Began work on the class.
'
'#############################################################################################

Public Class FuzzyColor
    Public ID As Integer = 1
    Public Name As String = "Undefined Color"

    Public Y As New FuzzyDouble(0, 1)
    Public U As New FuzzyDouble(0, 1)
    Public V As New FuzzyDouble(0, 1)


    '#########################################################################################
    'build a brand-new fuzzy color
    '#########################################################################################
    Sub New(ByVal iID As Integer, ByVal sName As String, ByVal dDeviation As Double)

        ID = iID
        Name = sName
        Me.SetDeviation(dDeviation)

    End Sub


    '#########################################################################################
    'assign a strict color to the probability values
    '#########################################################################################
    Public Sub SetColor(ByVal dY As Double, ByVal dU As Double, ByVal dV As Double)

        Y.Value = dY
        U.Value = dU
        V.Value = dV

    End Sub


    '#########################################################################################
    'assign a global property setting deviations
    '#########################################################################################
    Public Sub SetDeviation(ByVal dDev As Double)

        Y.Deviation = dDev
        U.Deviation = dDev
        V.Deviation = dDev

    End Sub


    '#########################################################################################
    'find out the probability match of the passed color against the real value. the total
    'probability of a color match is based on the corrected magnitude between the correct
    'color and the passed color
    '#########################################################################################
    Public Function Probability(ByVal dY As Double, ByVal dU As Double, ByVal dV As Double) As Double
        Dim dDev As FuzzyDouble
        Dim dChr As Double

        Dim dDY As Double
        Dim dDU As Double
        Dim dDV As Double
        Dim dCmpRel As Double

        'generate the differences between each set of points. the color and brightness sets
        'are considered to be two different sets; each has its own probability and influence
        dDY = (Y.Relevance * (Y.Value - dY)) ^ 2
        dDU = (U.Relevance * (U.Value - dU)) ^ 2
        dDV = (V.Relevance * (V.Value - dV)) ^ 2
        dCmpRel = Math.Sqrt(Y.Relevance ^ 2 + U.Relevance ^ 2 + V.Relevance ^ 2)

        'determine the composite of the three values
        dChr = Math.Sqrt(dDY + dDU + dDV) / dCmpRel

        'calculate the probability that the chrominances match.
        dDev = New FuzzyDouble(0, Y.Deviation)

        'return the resultant probability
        Return dDev.Probability(dChr)

    End Function

End Class