Option Explicit On 
Option Strict On


'#############################################################################################
' John Girard, Advanced Systems & Controls
'
' FuzzyDouble: this is a double-precision floating point, accurate to within a certain
' defined precision. The precision is assigned as the standard deviation, and the probability
' curve is Gaussian. That is, of a normal bell curve of data passed into Probability, 68% or
' so will be within one Deviation of the Value. The Probability function actually returns the
' actual Gaussian function value at the defined point.
'
' References:
' http://mathworld.wolfram.com/NormalDistribution.html
'
' Revision History:
' --------------------------------------------------------------------------------------------
'
' (01/25/05)    Began work on the class.
'
'#############################################################################################

Public Class FuzzyDouble
    Public Value As Double = 0.0
    Public Deviation As Double = 5.0

    'relevance is the amount of influence a component has on the final probability.
    Public Relevance As Double = 1.0


    '#########################################################################################
    'build a brand-new fuzzy number.
    '#########################################################################################
    Sub New(Optional ByVal dValue As Double = 0.0, Optional ByVal dDev As Double = 1.0)

        Value = dValue
        Deviation = dDev

    End Sub


    '#########################################################################################
    'find out the probability match of the passed number against real value
    '#########################################################################################
    Public Function Probability(ByVal dVal As Double) As Double
        Dim dResultant As Double

        'return the value of the probability function at the specified dVal. At dVal = Value,
        'the probability function will return 1 (the function has been normalized to this
        'concept
        dResultant = -((dVal - Value) ^ 2) / (2 * Deviation ^ 2)
        dResultant = Math.Exp(dResultant)
        Return dResultant

    End Function

End Class